package net.macpod.robocomm;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

/**
 * Controls interface of app to allow users to start/stop robot client socket
 * connections.
 * 
 * @author Jeffrey Nelson <nelsonjm@macpod.net> http://macpod.net
 * @date 1/20/09
 * @version 1.0
 * Licensed under the GPL v2, see the file LICENSE in this tarball.
 */
public class RoboComm extends Activity {
	private RoboClient engine;
	protected RoboComm ref = this;

	/**
	 * Sets up view and state of interface to disabled.
	 */
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		engine = new RoboClient();

		Button stateButton = (Button) findViewById(R.id.state_button);
		stateButton.setOnClickListener(stateListener);

		disconnectedInterface();
	}

	/**
	 * Disconnects from robot controller server when the app is closed.
	 */
	protected void onStop() {
		engine.disconnect();
		super.onStop();
	}

	/**
	 * Listens to when the start/stop button is clicked and toggles
	 * if the server is running or not.
	 */
	private OnClickListener stateListener = new OnClickListener() {
		public void onClick(View v) {
			try {
				if (engine.connected()) {
					// Disconnect
					engine.disconnect();
				} else {
					// Connect
					String server = ((EditText) findViewById(R.id.server_field))
							.getText().toString();
					int port = Integer
							.parseInt(((EditText) findViewById(R.id.port_field))
									.getText().toString());

					if (!engine.connect(server, port, interface_handler)) {
						// Report a vague error for users to ponder over if we 
						// can't connect.
						Dialogue("Could not connect to given server.");
					}
				}
			} catch (Exception e) {
				Dialogue("An error occured.");

			}
		}

	};

	public static final int CONNECTED_INTERFACE = 0;
	public static final int DISCONNECTED_INTERFACE = 1;

	/**
	 * This handler is used so that the robot client thread can change the user interface.
	 */
	private Handler interface_handler = new Handler() {
		public void handleMessage(Message msg) {
			switch (msg.what) {
			case CONNECTED_INTERFACE:
				connectedInterface();
				break;
			case DISCONNECTED_INTERFACE:
				disconnectedInterface();
				break;
			}
		}
	};

	/**
	 * Changes the interface to reflect that we are connected to the robot
	 * control server.
	 */
	public void connectedInterface() {
		try {
			TextView tv = (TextView) findViewById(R.id.status_label);
			tv.setText(R.string.status_running);
			Button stateButton = (Button) findViewById(R.id.state_button);
			stateButton.setText(R.string.state_stop);
		} catch (Exception e) {
		}

	}

	/**
	 * Changes the interface to reflect we are not connected to a robot
	 * control server.
	 */
	public void disconnectedInterface() {
		try {
			TextView tv = (TextView) findViewById(R.id.status_label);
			tv.setText(R.string.status_stopped);
			Button stateButton = (Button) findViewById(R.id.state_button);
			stateButton.setText(R.string.state_start);
		} catch (Exception e) {
		}
	}

	/**
	 * Convenience method used to pop up a dialogue window to alert the user of 
	 * something, possibly magical.
	 * @param message Message text to report to user.
	 */
	private void Dialogue(CharSequence message) {
		new AlertDialog.Builder(RoboComm.this).setMessage(message)
				.setNeutralButton("OK", new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
					}
				}).show();
	}

}
